#include "ds_vk_rectangle.hpp"
#include "print_draw_strategy.hpp"
#include "vulkan_instance.hpp"
#include "renderer2D.hpp"

#include "renderer3D.hpp"
#include "vulkan_settings.hpp"
#include "font.hpp"

#include <chrono>
#include <ratio>
#include <thread>

namespace gz::vlk {
    int mainLoop() {
        Log log(LogCreateInfo {
        	.logfile = "main.log",
        	.storeLog = false,
        	.prefix = "Main",
        	.prefixColor = Color::BG_RED,
        	.timeColor = settings::VULKAN_MESSAGE_TIME_COLOR,
        });

        gz::SettingsManagerCreateInfo<VULKAN_SETTINGS_MAN_TYPES> smCI {
        	.filepath = settings::CONFIG_FILE,
        	.initialValues = settings::INITIAL_SETTINGS,
        	.readFileOnCreation = true,
        	.writeFileOnExit = true,
        	.throwExceptionWhenNewValueNotAllowed = true,
        };

        log("Creating instance");
        VulkanInstance vulkanInstance(smCI);
        vulkanInstance.init();
        TextureManager textureManager(vulkanInstance);
        Renderer2D r2D(vulkanInstance, textureManager);

        log("Startup complete. Drawing shapes");

        BufferManager<Vertex2D, uint32_t> bufferManager(vulkanInstance);
        auto createRect = [&r2D, &bufferManager](auto posX, auto posY, auto width, auto height, const auto& texture) {
            return Rectangle(posX, posY, width, height, std::make_unique<RectangleTexturedVkDS>(RectangleTexturedVkDS(r2D, bufferManager, texture)));
            /* return Rectangle(posX, poxY, width, height, std::make_unique<PrintDrawStrategy<Rectangle>>()); */
        };
        std::vector<Rectangle> rects;
        rects.emplace_back(createRect(    0,   0,  48,  48, "blocks/leaves.png"));
        rects.emplace_back(createRect( -190,-191, 192, 193, "blocks/crate.png"));
        rects.emplace_back(createRect(  32,  120, 400, 400, "blocks/dirt.png"));
        rects.emplace_back(createRect(  90,   91,  92,  93, "items/sword.png"));
        rects.emplace_back(createRect( 190,  191, 192, 193, "items/crossbow.png"));
        rects.emplace_back(createRect(-600, -600, 800, 400, "blocks/ice.png"));
        rects.emplace_back(createRect(  32,  400, 400, 100, "special/hotbar.png"));
        rects.emplace_back(createRect(-600, -600, 800, 400, "special/func_nocol.png"));
        rects.emplace_back(createRect(-200, -400, 200, 200, "entities/sheep.png"));
        rects.emplace_back(createRect(-600,    0, 600, 600, "atlas_0"));
        log("Initializing complete.");

        /* Renderer3D r3D(vulkanInstance, textureManager); */
        /* gz::vlk::FontManager fm(vulkanInstance, "fonts"); */
        /* fm.loadFont("menu.ttf"); */
        /* fm.getFaces().at("menu.ttf"). */
        /* try */ 
        {
            uint32_t imageIndex;
            /* std::chrono::time_point now = std::chrono::system_clock::now(); */
            bool first2d = false;
            int count = 0;
            while (!glfwWindowShouldClose(vulkanInstance.getWindow())) {
                glfwPollEvents();
                imageIndex = vulkanInstance.beginFrameDraw();
                uint32_t currentFrame = vulkanInstance.getCurrentFrame();
                r2D.beginFrameDraw(imageIndex, currentFrame);
                for (auto& shape : rects) {
                    shape.draw();
                }
                if (count > 100) {
                    /* r2D.drawFrame(imageIndex); */
                    r2D.endFrameDraw(imageIndex, currentFrame);
                    /* r3D.drawFrame(imageIndex); */
                }
                else {
                    /* r3D.drawFrame(imageIndex); */
                    r2D.endFrameDraw(imageIndex, currentFrame);
                }
                count = count < 200 ? ++count : 0;
                /* first2d = !first2d; */
                vulkanInstance.endFrameDraw(imageIndex);
                auto SLEEP_TIME = std::chrono::milliseconds(1000 / vulkanInstance.getSettings().get<uint32_t>("framerate"));
                std::this_thread::sleep_for(SLEEP_TIME);
                /* std::chrono::time_point now2 = std::chrono::system_clock::now(); */
                /* std::chrono::nanoseconds dur = std::chrono::duration_cast<std::chrono::nanoseconds>(now2 - now); */
                /* std::cout << "Frametime: (ns)" << dur.count() << " in fps: " << 1e9 / (dur.count()) << "\n"; */
                /* now = now2; */
            }
            vulkanInstance.cleanup();
        }
         /* catch (const std::exception& e) { */
         /*     std::cerr << e.what() << std::endl; */
         /*     return 1; */
        /* } */

        return 0;
    }
}


int main() {
    return gz::vlk::mainLoop();
}
