#include "ds_vk_rectangle.hpp"

#include "texture_manager.hpp"
#include "renderer2D.hpp"

namespace gz::vlk {

	RectangleTexturedVkDS::RectangleTexturedVkDS(Renderer2D& renderer, BufferManager<Vertex2D, uint32_t>& bufferManager, const std::string& textureName)
        : VulkanDS(renderer, bufferManager), 
            VulkanTexturedDS(this->renderer.get().getTextureManager(), this->renderer.get().getTextureManager().getTexture(textureName)) 
    {

    }


    void RectangleTexturedVkDS::update(const Rectangle& rect) {
        // update buffers
        vertices.clear();
        const auto& pos = rect.getPosition();
        const auto& size = rect.getSize();
        glm::vec3 color(0, 0, 0);
        // topleft, topright, botright, botleft -> clockwise
        vertices.emplace_back(Vertex2D{pos, color, glm::vec2(0, 0)});
        vertices.emplace_back(Vertex2D{glm::vec2(pos.x + size.x, pos.y), color, glm::vec2(1, 0)});
        vertices.emplace_back(Vertex2D{glm::vec2(pos.x + size.x, pos.y + size.y), color, glm::vec2(1, 1)});
        vertices.emplace_back(Vertex2D{glm::vec2(pos.x, pos.y + size.y), color, glm::vec2(0, 1)});
        /* indices = { 0, 1, 2, 2, 3, 0 }; */
        /* indices = { 2, 1, 0, 2, 3, 0 }; */
        indices = { 0, 2, 1, 0, 3, 2 };
        normalizeVertices(renderer.get().getScExtent().width, renderer.get().getScExtent().height);

        // update texture
        TextureManager& tm = textureManager.get();
        tm.setTexCoords(texture, vertices[0].texCoord);
        tm.setTexCoords(texture, vertices[1].texCoord);
        tm.setTexCoords(texture, vertices[2].texCoord);
        tm.setTexCoords(texture, vertices[3].texCoord);

        updateBufferInfos();
    }

	void RectangleTexturedVkDS::draw() {
        renderer.get().drawShape(bufferManager.get(), vertexBufferInfo, indexBufferInfo, textureAtlasIndex);
	}
}  // namespace gz::vlk
